#
# Un modelo de un cubo proyecto
# segn el modelo de la cmara obscura
#
# Fraga 16/12/2013
# 
# Los puntos de salida pueden graficarse con gnuplot
#

# Datos de la cmara:
$c = [ 4, 2, 6 ]; # Posicin de la cmara
# $a = [ -1, 1, 0 ];  # Direccin de arriba para la cmara
$a = [ 0, 1, 0 ];  # Direccin de arriba para la cmara
$p = [ 0, 0, 0 ];  # Punto en la escena hacia donde ve la camara 

# La distancia focal y el punto principal
$f = 1000;
$u0 = 300;
$v0 = 200;

# La escena, un simple cubo.
$cubo = [
[ 0, 0, 1 ],
[ 1, 0, 1 ],
[ 1, 1, 1 ],
[ 0, 1, 1 ],
[ 0, 0, 0 ],
[ 1, 0, 0 ],
[ 1, 1, 0 ],
[ 0, 1, 0 ] ];

# Hacemos la matriz R
$ez = [];
$ez->[0] = $c->[0] - $p[0];
$ez->[1] = $c->[1] - $p[1];
$ez->[2] = $c->[2] - $p[2];

$norma = sqrt( $ez->[0]*$ez->[0] + $ez->[1]*$ez->[1] + $ez->[2]*$ez->[2] );
$ez->[0] /= $norma;
$ez->[1] /= $norma;
$ez->[2] /= $norma;

# ex = u x ez / (| u x ez | )
#   i  j  k |  i  j 
#  u0 u1 u2 | u0 u1 = i(u1*v2-v1*u2), j(u2*v0-v2*u0), k(u0*v1 - v0*u1)
#  v0 v1 v2 | v0 v1
# 
$ex = [];
$ex->[0] = $a->[1]*$ez->[2] - $a->[2]*$ez->[1];
$ex->[1] = $a->[2]*$ez->[0] - $a->[0]*$ez->[2];
$ex->[2] = $a->[0]*$ez->[1] - $a->[1]*$ez->[0];
$norma = sqrt( $ex->[0]*$ex->[0] + $ex->[1]*$ex->[1] + $ex->[2]*$ex->[2] );
$ex->[0] /= $norma;
$ex->[1] /= $norma;
$ex->[2] /= $norma;

# ey = ez x ex 
$ey = [];
$ey->[0] = $ez->[1]*$ex->[2] - $ez->[2]*$ex->[1];
$ey->[1] = $ez->[2]*$ex->[0] - $ez->[0]*$ex->[2];
$ey->[2] = $ez->[0]*$ex->[1] - $ez->[1]*$ex->[0];
$norma = sqrt( $ey->[0]*$ey->[0] + $ey->[1]*$ey->[1] + $ey->[2]*$ey->[2] );
$ey->[0] /= $norma;
$ey->[1] /= $norma;
$ey->[2] /= $norma;



# y proyectamos cada punto segn K, R, y c

# $R = [ [1,0,0], [0,1,0], [0,0,1] ];
$R = [ [ $ex->[0],$ex->[1],$ex->[2] ], [ $ey->[0],$ey->[1],$ey->[2] ], [$ez->[0],$ez->[1],$ez->[2]] ];
$K = [ [$f,0,$u0], [0,$f,$v0], [0,0,-1] ];

traza_cubo( $cubo, $K, $R, $c );

sub traza_cubo( $$$$ )
{
	my $punto = shift;
	my $K = shift;
	my $R = shift;
	my $c = shift;

	my $pt = [];

	# Translados cada punto
	for( my $i=0; $i<8; $i++ ) {
		$pt->[$i][0] = $punto->[$i][0] - $c->[0];
		$pt->[$i][1] = $punto->[$i][1] - $c->[1];
		$pt->[$i][2] = $punto->[$i][2] - $c->[2];
	}
	#print "Puntos transladados\n";
	#for( my $i=0; $i<8; $i++ ) {
	#	print "$pt->[$i][0] $pt->[$i][1] $pt->[$i][2]\n";
	#}
	my $r11 = $R->[0][0];
	my $r12 = $R->[0][1];
	my $r13 = $R->[0][2];

	my $r21 = $R->[1][0];
	my $r22 = $R->[1][1];
	my $r23 = $R->[1][2];

	my $r31 = $R->[2][0];
	my $r32 = $R->[2][1];
	my $r33 = $R->[2][2];

	# Rotamos todos los puntos
	my $pr = [];
	for( my $i=0; $i<8; $i++ ) {
		$pr->[$i][0] = $r11 * $pt->[$i][0] + $r12 * $pt->[$i][1] + $r13 * $pt->[$i][2];
		$pr->[$i][1] = $r21 * $pt->[$i][0] + $r22 * $pt->[$i][1] + $r23 * $pt->[$i][2];
		$pr->[$i][2] = $r31 * $pt->[$i][0] + $r32 * $pt->[$i][1] + $r33 * $pt->[$i][2];
	}
	#print "Puntos rotados\n";
	#for( my $i=0; $i<8; $i++ ) {
	#	print "$pr->[$i][0] $pr->[$i][1] $pr->[$i][2]\n";
	#}

	# Los proyectamos
	my $f = $K->[0][0];
	my $u0 = $K->[0][2];
	my $v0 = $K->[1][2];
	for( my $i=0; $i<8; $i++ ) {
		if ( $pr->[$i][2] > 0 ) {
			print "Ojo: El punto $i es positivo y no se implementado el recorte segn el plano xy\n";
		}
		$pt->[$i][0] = $f*$pr->[$i][0]/(-$pr->[$i][2]) + $u0;
		$pt->[$i][1] = $f*$pr->[$i][1]/(-$pr->[$i][2]) + $v0;
	}

	# Los dibujo
	# Puntos 0, 1, 2, 3
	my $x;
	my $y;
	for( my $i=0; $i<4; $i++ ) {
		$x = $pt->[$i][0];
		$y = $pt->[$i][1];
		print "$x $y\n";
	}
	$x = $pt->[0][0];
	$y = $pt->[0][1];
	print "$x $y\n";
	print "\n";

	# Puntos 4, 5, 6, 7
	for( my $i=4; $i<8; $i++ ) {
		$x = $pt->[$i][0];
		$y = $pt->[$i][1];
		print "$x $y\n";
	}
	$x = $pt->[4][0];
	$y = $pt->[4][1];
	print "$x $y\n";
	print "\n";

	# puntos 0 4
	$x = $pt->[0][0];
	$y = $pt->[0][1];
	print "$x $y\n";
	$x = $pt->[4][0];
	$y = $pt->[4][1];
	print "$x $y\n";
	print "\n";

	# puntos 1 5
	$x = $pt->[1][0];
	$y = $pt->[1][1];
	print "$x $y\n";
	$x = $pt->[5][0];
	$y = $pt->[5][1];
	print "$x $y\n";
	print "\n";

	# puntos 2 6
	$x = $pt->[2][0];
	$y = $pt->[2][1];
	print "$x $y\n";
	$x = $pt->[6][0];
	$y = $pt->[6][1];
	print "$x $y\n";
	print "\n";

	# puntos 3 7
	$x = $pt->[3][0];
	$y = $pt->[3][1];
	print "$x $y\n";
	$x = $pt->[7][0];
	$y = $pt->[7][1];
	print "$x $y\n";
	print "\n";
}

